#ifndef CONST_H
#define CONST_H

#define SNR_UNIT 0.25                      /* SNR unit (dBHz) */

#define MAXFREQ 7                          //! max NFREQ 
#define SYS_NONE 0  //!0x00                //! navigation system: none 
#define SYS_GPS 1   //!0x01                //! navigation system: GPS 
#define SYS_SBS 2   //!0x02                //! navigation system: SBAS 
#define SYS_GLO 4   //!0x04                //! navigation system: GLONASS 
#define SYS_GAL 8   //!0x08                //! navigation system: Galileo 
#define SYS_QZS 16  //!0x10                //! navigation system: QZSS 
#define SYS_CMP 32  //!0x20                //! navigation system: BeiDou 
#define SYS_IRN 64  //!0x40                //! navigation system: IRNS 
#define SYS_LEO 128 //!0x80                //! navigation system: LEO 
#define SYS_ALL 255 //!0xFF                //! navigation system: all 

#define TSYS_GPS 0                         //! time system: GPS time 
#define TSYS_UTC 1                         //! time system: UTC 
#define TSYS_GLO 2                         //! time system: GLONASS time 
#define TSYS_GAL 3                         //! time system: Galileo time 
#define TSYS_QZS 4                         //! time system: QZSS time 
#define TSYS_CMP 5                         //! time system: BeiDou time 
#define TSYS_IRN 6                         //! time system: IRNSS time 

#define NFREQ 3                            //! number of carrier frequencies 
#define NFREQGLO 2                         //! number of carrier frequencies of GLONASS 
#define NEXOBS 3                           //! number of extended obs codes 

#define MINPRNGPS 1                        //! min satellite PRN number of GPS 
#define MAXPRNGPS 32                       //! max satellite PRN number of GPS 
#define NSATGPS (MAXPRNGPS-MINPRNGPS+1)    //! number of GPS satellites 
#define NSYSGPS 1

#define MINPRNGLO 1                        //! min satellite slot number of GLONASS 
#define MAXPRNGLO 27                       //! max satellite slot number of GLONASS 
#define NSATGLO (MAXPRNGLO-MINPRNGLO+1)    //! number of GLONASS satellites 
#define NSYSGLO 1

#define MINPRNGAL 1                        //! min satellite PRN number of Galileo 
#define MAXPRNGAL 36                       //! max satellite PRN number of Galileo 
#define NSATGAL (MAXPRNGAL-MINPRNGAL+1)    //! number of Galileo satellites 
#define NSYSGAL 1

#define MINPRNQZS 193                      //! min satellite PRN number of QZSS 
#define MAXPRNQZS 202                      //! max satellite PRN number of QZSS 
#define MINPRNQZS_S 183                    //! min satellite PRN number of QZSS SAIF 
#define MAXPRNQZS_S 191                    //! max satellite PRN number of QZSS SAIF 
#define NSATQZS (MAXPRNQZS-MINPRNQZS+1)    //! number of QZSS satellites 
#define NSYSQZS 1

#define MINPRNCMP 1                        //! min satellite sat number of BeiDou 
#define MAXPRNCMP 37                       //! max satellite sat number of BeiDou 
#define NSATCMP (MAXPRNCMP-MINPRNCMP+1)    //! number of BeiDou satellites 
#define NSYSCMP 1

#define MINPRNIRN 1                        //! min satellite sat number of IRNSS 
#define MAXPRNIRN 7                        //! max satellite sat number of IRNSS 
#define NSATIRN (MAXPRNIRN-MINPRNIRN+1)    //! number of IRNSS satellites 
#define NSYSIRN 1

#define MINPRNLEO 1                        //! min satellite sat number of LEO 
#define MAXPRNLEO 10                       //! max satellite sat number of LEO 
#define NSATLEO (MAXPRNLEO-MINPRNLEO+1)    //! number of LEO satellites 
#define NSYSLEO 1

#define NSYS (NSYSGPS+NSYSGLO+NSYSGAL+NSYSQZS+NSYSCMP+NSYSIRN+NSYSLEO) //! number of systems

#define MINPRNSBS 120                      //! min satellite PRN number of SBAS 
#define MAXPRNSBS 142                      //! max satellite PRN number of SBAS 
#define NSATSBS (MAXPRNSBS-MINPRNSBS+1)    //! number of SBAS satellites 
#define MAXSAT (NSATGPS+NSATGLO+NSATGAL+NSATQZS+NSATCMP+NSATIRN+NSATSBS+NSATLEO)
                                           //! max satellite number (1 to MAXSAT) 
#define MAXSTA 255

#define MAXOBS 128                         //! max number of obs in an epoch
#define MAXRCV 64                          //! max receiver number (1 to MAXRCV)
#define MAXOBSTYPE 64                      //! max number of obs type in RINEX

#define MAXEXFILE 1024                     //! max number of expanded files 

#define MAXSBSURA 8                        //! max URA of SBAS satellite 
#define MAXBAND 10                         //! max SBAS band of IGP 
#define MAXNIGP 201                        //! max number of IGP in SBAS band 
#define MAXNGEO 4                          //! max number of GEO satellites 
#define MAXCOMMENT 10                      //! max number of RINEX comments 
#define MAXSTRPATH 1024                    //! max length of stream path 
#define MAXSTRMSG 1024                     //! max length of stream message 
#define MAXSTRRTK 8                        //! max number of stream in RTK server 
#define MAXSBSMSG 32                       //! max number of SBAS msg in RTK server 
#define MAXSOLMSG 8191                     //! max length of solution message 
#define MAXRAWLEN 4096                     //! max length of receiver raw message 
#define MAXERRMSG 4096                     //! max length of error/warning message 
#define MAXANT 64                          //! max length of station name/antenna type 
#define MAXSOLBUF 256                      //! max number of solution buffer 
#define MAXSOLNUM 86400                    //! max number of solution in program
#define MAXOBSBUF 128                      //! max number of observation data buffer 
#define MAXNRPOS 16                        //! max number of reference positions 
#define MAXLEAPS 64                        //! max number of leap seconds table 
#define MAXGISLAYER 32                     //! max number of GIS data layers 
#define MAXRCVCMD 4096                     //! max length of receiver commands 

#define OBSTYPE_PR 1     //!#01            //! observation type: pseudorange 
#define OBSTYPE_CP 2     //!#02            //! observation type: carrier-phase 
#define OBSTYPE_DOP 4    //!#04            //! observation type: doppler-freq 
#define OBSTYPE_SNR 8    //!#08            //! observation type: SNR 
#define OBSTYPE_ALL 255  //!#FF            //! observation type: all 

#define FREQTYPE_L1 1    //!0x01           //! frequency type: L1/E1 
#define FREQTYPE_L2 2    //!0x02           //! frequency type: L2/B1 
#define FREQTYPE_L5 4    //!0x04           //! frequency type: L5/E5a/L3 
#define FREQTYPE_L6 8    //!0x08           //! frequency type: E6/LEX/B3 
#define FREQTYPE_L7 16   //!0x10           //! frequency type: E5b/B2 
#define FREQTYPE_L8 32   //!0x20           //! frequency type: E5(a+b) 
#define FREQTYPE_L9 64   //!0x40           //! frequency type: S 
#define FREQTYPE_ALL 255 //!0xFF           //! frequency type: all 

#define CODE_NONE 0                        //! obs code: none or unknown 
#define CODE_L1C 1                         //! obs code: L1C/A,G1C/A,E1C (GPS,GLO,GAL,QZS,SBS) 
#define CODE_L1P 2                         //! obs code: L1P,G1P    (GPS,GLO) 
#define CODE_L1W 3                         //! obs code: L1 Z-track (GPS) 
#define CODE_L1Y 4                         //! obs code: L1Y        (GPS) 
#define CODE_L1M 5                         //! obs code: L1M        (GPS) 
#define CODE_L1N 6                         //! obs code: L1codeless (GPS) 
#define CODE_L1S 7                         //! obs code: L1C(D)     (GPS,QZS) 
#define CODE_L1L 8                         //! obs code: L1C(P)     (GPS,QZS) 
#define CODE_L1E 9                         //! (not used) 
#define CODE_L1A 10                        //! obs code: E1A        (GAL) 
#define CODE_L1B 11                        //! obs code: E1B        (GAL) 
#define CODE_L1X 12                        //! obs code: E1B+C,L1C(D+P) (GAL,QZS) 
#define CODE_L1Z 13                        //! obs code: E1A+B+C,L1SAIF (GAL,QZS) 
#define CODE_L2C 14                        //! obs code: L2C/A,G1C/A (GPS,GLO) 
#define CODE_L2D 15                        //! obs code: L2 L1C/A-(P2-P1) (GPS) 
#define CODE_L2S 16                        //! obs code: L2C(M)     (GPS,QZS) 
#define CODE_L2L 17                        //! obs code: L2C(L)     (GPS,QZS) 
#define CODE_L2X 18                        //! obs code: L2C(M+L),B1I+Q (GPS,QZS,CMP) 
#define CODE_L2P 19                        //! obs code: L2P,G2P    (GPS,GLO) 
#define CODE_L2W 20                        //! obs code: L2 Z-track (GPS) 
#define CODE_L2Y 21                        //! obs code: L2Y        (GPS) 
#define CODE_L2M 22                        //! obs code: L2M        (GPS) 
#define CODE_L2N 23                        //! obs code: L2codeless (GPS) 
#define CODE_L5I 24                        //! obs code: L5/E5aI    (GPS,GAL,QZS,SBS) 
#define CODE_L5Q 25                        //! obs code: L5/E5aQ    (GPS,GAL,QZS,SBS) 
#define CODE_L5X 26                        //! obs code: L5/E5aI+Q/L5B+C (GPS,GAL,QZS,IRN,SBS) 
#define CODE_L7I 27                        //! obs code: E5bI,B2I   (GAL,CMP) 
#define CODE_L7Q 28                        //! obs code: E5bQ,B2Q   (GAL,CMP) 
#define CODE_L7X 29                        //! obs code: E5bI+Q,B2I+Q (GAL,CMP) 
#define CODE_L6A 30                        //! obs code: E6A        (GAL) 
#define CODE_L6B 31                        //! obs code: E6B        (GAL) 
#define CODE_L6C 32                        //! obs code: E6C        (GAL) 
#define CODE_L6X 33                        //! obs code: E6B+C,LEXS+L,B3I+Q (GAL,QZS,CMP) 
#define CODE_L6Z 34                        //! obs code: E6A+B+C    (GAL) 
#define CODE_L6S 35                        //! obs code: LEXS       (QZS) 
#define CODE_L6L 36                        //! obs code: LEXL       (QZS) 
#define CODE_L8I 37                        //! obs code: E5(a+b)I   (GAL) 
#define CODE_L8Q 38                        //! obs code: E5(a+b)Q   (GAL) 
#define CODE_L8X 39                        //! obs code: E5(a+b)I+Q (GAL) 
#define CODE_L2I 40                        //! obs code: B1I        (BDS) 
#define CODE_L2Q 41                        //! obs code: B1Q        (BDS) 
#define CODE_L6I 42                        //! obs code: B3I        (BDS) 
#define CODE_L6Q 43                        //! obs code: B3Q        (BDS) 
#define CODE_L3I 44                        //! obs code: G3I        (GLO) 
#define CODE_L3Q 45                        //! obs code: G3Q        (GLO) 
#define CODE_L3X 46                        //! obs code: G3I+Q      (GLO) 
#define CODE_L1I 47                        //! obs code: B1I        (BDS) 
#define CODE_L1Q 48                        //! obs code: B1Q        (BDS) 
#define CODE_L5A 49                        //! obs code: L5A SPS    (IRN) 
#define CODE_L5B 50                        //! obs code: L5B RS(D)  (IRN) 
#define CODE_L5C 51                        //! obs code: L5C RS(P)  (IRN) 
#define CODE_L9A 52                        //! obs code: SA SPS     (IRN) 
#define CODE_L9B 53                        //! obs code: SB RS(D)   (IRN) 
#define CODE_L9C 54                        //! obs code: SC RS(P)   (IRN) 
#define CODE_L9X 55                        //! obs code: SB+C       (IRN) 
#define MAXCODE 55                         //! max number of obs code 

#define PMODE_SINGLE 0                     //! positioning mode: single 
#define PMODE_DGPS 1                       //! positioning mode: DGPS/DGNSS 
#define PMODE_KINEMA 2                     //! positioning mode: kinematic 
#define PMODE_STATIC 3                     //! positioning mode: static 
#define PMODE_MOVEB 4                      //! positioning mode: moving-base 
#define PMODE_FIXED 5                      //! positioning mode: fixed 
#define PMODE_PPP_KINEMA 6                 //! positioning mode: PPP-kinemaric 
#define PMODE_PPP_STATIC 7                 //! positioning mode: PPP-static 
#define PMODE_PPP_FIXED 8                  //! positioning mode: PPP-fixed 

#define SOLF_LLH 0                         //! solution format: lat/lon/height 
#define SOLF_XYZ 1                         //! solution format: x/y/z-ecef 
#define SOLF_ENU 2                         //! solution format: e/n/u-baseline 
#define SOLF_NMEA 3                        //! solution format: NMEA-183 
#define SOLF_STAT 4                        //! solution format: solution status 
#define SOLF_GSIF 5                        //! solution format: GSI F1/F2 

#define SOLQ_NONE 0                        //! solution status: no solution 
#define SOLQ_FIX 1                         //! solution status: fix 
#define SOLQ_FLOAT 2                       //! solution status: float 
#define SOLQ_SBAS 3                        //! solution status: SBAS 
#define SOLQ_DGPS 4                        //! solution status: DGPS/DGNSS 
#define SOLQ_SINGLE 5                      //! solution status: single 
#define SOLQ_PPP 6                         //! solution status: PPP 
#define SOLQ_DR 7                          //! solution status: dead reconing 
#define MAXSOLQ 7                          //! max number of solution status 

#define TIMES_GPST 0                       //! time system: gps time 
#define TIMES_UTC 1                        //! time system: utc 
#define TIMES_JST 2                        //! time system: jst 

#define IONOOPT_OFF 0                      //! ionosphere option: correction off 
#define IONOOPT_BRDC 1                     //! ionosphere option: broadcast model 
#define IONOOPT_SBAS 2                     //! ionosphere option: SBAS model 
#define IONOOPT_IFLC 3                     //! ionosphere option: L1/L2 or L1/L5 iono-free LC 
#define IONOOPT_EST 4                      //! ionosphere option: estimation 
#define IONOOPT_TEC 5                      //! ionosphere option: IONEX TEC model 
#define IONOOPT_QZS 6                      //! ionosphere option: QZSS broadcast model 
#define IONOOPT_LEX 7                      //! ionosphere option: QZSS LEX ionospehre 
#define IONOOPT_STEC 8                     //! ionosphere option: SLANT TEC model 

#define TROPOPT_OFF 0                      //! troposphere option: correction off 
#define TROPOPT_SAAS 1                     //! troposphere option: Saastamoinen model 
#define TROPOPT_SBAS 2                     //! troposphere option: SBAS model 
#define TROPOPT_EST 3                      //! troposphere option: ZTD estimation 
#define TROPOPT_ESTG 4                     //! troposphere option: ZTD+grad estimation 
#define TROPOPT_ZTD 5                      //! troposphere option: ZTD correction 

#define EPHOPT_BRDC 0                      //! ephemeris option: broadcast ephemeris 
#define EPHOPT_PREC 1                      //! ephemeris option: precise ephemeris 
#define EPHOPT_SBAS 2                      //! ephemeris option: broadcast + SBAS 
#define EPHOPT_SSRAPC 3                    //! ephemeris option: broadcast + SSR_APC 
#define EPHOPT_SSRCOM 4                    //! ephemeris option: broadcast + SSR_COM 
#define EPHOPT_LEX 5                       //! ephemeris option: QZSS LEX ephemeris 

#define ARMODE_OFF 0                       //! AR mode: off 
#define ARMODE_CONT 1                      //! AR mode: continuous 
#define ARMODE_INST 2                      //! AR mode: instantaneous 
#define ARMODE_FIXHOLD 3                   //! AR mode: fix and hold 
#define ARMODE_WLNL 4                      //! AR mode: wide lane/narrow lane 
#define ARMODE_TCAR 5                      //! AR mode: triple carrier ar 

#define SBSOPT_LCORR 1                     //! SBAS option: long term correction 
#define SBSOPT_FCORR 2                     //! SBAS option: fast correction 
#define SBSOPT_ICORR 4                     //! SBAS option: ionosphere correction 
#define SBSOPT_RANGE 8                     //! SBAS option: ranging 

#define POSOPT_POS 0                       //! pos option: LLH/XYZ 
#define POSOPT_SINGLE 1                    //! pos option: average of single pos 
#define POSOPT_FILE 2                      //! pos option: read from pos file 
#define POSOPT_RINEX 3                     //! pos option: rinex header pos 
#define POSOPT_RTCM 4                      //! pos option: rtcm station pos 
#define POSOPT_RAW 5                       //! pos option: raw station pos 

#define STR_NONE 0                         //! stream type: none 
#define STR_SERIAL 1                       //! stream type: serial 
#define STR_FILE 2                         //! stream type: file 
#define STR_TCPSVR 3                       //! stream type: TCP server 
#define STR_TCPCLI 4                       //! stream type: TCP client 
#define STR_NTRIPSVR 6                     //! stream type: NTRIP server 
#define STR_NTRIPCLI 7                     //! stream type: NTRIP client 
#define STR_FTP 8                          //! stream type: ftp 
#define STR_HTTP 9                         //! stream type: http 
#define STR_NTRIPC_S 10                    //! stream type: NTRIP caster server 
#define STR_NTRIPC_C 11                    //! stream type: NTRIP caster client 
#define STR_UDPSVR 12                      //! stream type: UDP server 
#define STR_UDPCLI 13                      //! stream type: UDP server 
#define STR_MEMBUF 14                      //! stream type: memory buffer 

#define STRFMT_RTCM2 0                     //! stream format: RTCM 2 
#define STRFMT_RTCM3 1                     //! stream format: RTCM 3 
#define STRFMT_OEM4 2                      //! stream format: NovAtel OEMV/4 
#define STRFMT_OEM3 3                      //! stream format: NovAtel OEM3 
#define STRFMT_UBX 4                       //! stream format: u-blox LEA-*T 
#define STRFMT_SS2 5                       //! stream format: NovAtel Superstar II 
#define STRFMT_CRES 6                      //! stream format: Hemisphere 
#define STRFMT_STQ 7                       //! stream format: SkyTraq S1315F 
#define STRFMT_GW10 8                      //! stream format: Furuno GW10 
#define STRFMT_JAVAD 9                     //! stream format: JAVAD GRIL/GREIS 
#define STRFMT_NVS 10                      //! stream format: NVS NVC08C 
#define STRFMT_BINEX 11                    //! stream format: BINEX 
#define STRFMT_RT17 12                     //! stream format: Trimble RT17 
#define STRFMT_SEPT 13                     //! stream format: Septentrio 
#define STRFMT_CMR  14                     //! stream format: CMR/CMR+ 
#define STRFMT_TERSUS 15                   //! stream format: TERSUS 
#define STRFMT_LEXR 16                     //! stream format: Furuno LPY-10000 
#define STRFMT_RINEX 17                    //! stream format: RINEX 
#define STRFMT_SP3 18                      //! stream format: SP3 
#define STRFMT_RNXCLK 19                   //! stream format: RINEX CLK 
#define STRFMT_SBAS 20                     //! stream format: SBAS messages 
#define STRFMT_NMEA 21                     //! stream format: NMEA 0183 
#define MAXRCVFMT 15                       //! max number of receiver format 

#define STR_MODE_R 1                       //! stream mode: read 
#define STR_MODE_W 2                       //! stream mode: write 
#define STR_MODE_RW 3                      //! stream mode: read/write 

#define GEOID_EMBEDDED 0                   //! geoid model: embedded geoid 
#define GEOID_EGM96_M150 1                 //! geoid model: EGM96 15x15" 
#define GEOID_EGM2008_M25 2                //! geoid model: EGM2008 2.5x2.5" 
#define GEOID_EGM2008_M10 3                //! geoid model: EGM2008 1.e0x1.e0" 
#define GEOID_GSI2000_M15 4                //! geoid model: GSI geoid 2000 1.e0x1.5" 
#define GEOID_RAF09 5                      //! geoid model: IGN RAF09 for France 1.5"x2" 

#define DLOPT_FORCE 1                      //! download option: force download existing
#define DLOPT_KEEPCMP 2                    //! download option: keep compressed file
#define DLOPT_HOLDERR 4                    //! download option: hold on error file
#define DLOPT_HOLDLST 8                    //! download option: hold on listing file

#define LLI_SLIP 1                         //! LLI: cycle-slip
#define LLI_HALFC 2                        //! LLI: half-cycle not resovled
#define LLI_BOCTRK 4                       //! LLI: boc tracking of mboc signal
#define LLI_HALFA 64                       //! LLI: half-cycle added
#define LLI_HALFS 128                      //! LLI: half-cycle subtracted
#define IMUFMT_KVH 1                       //! imu data format KVH

#define FPOUT 10                           //! solution files  (except for 1/2/5/6)
#define FPREAD 11                          //! obs/nav  files
#define FPOPT 12                           //! option   files
#define FPADD 13                           //! additional files
#define FPCLK 14                           //! additional files (sat clock bias)
#define FPPOS 15                           //! additional files (sat pos)

#define MAX_ITER_KEPLER 30                 //! max number of iteration of Kelpler 

#define NX (4+3)                           //! # of estimated parameters 
#define MAXITR 10                          //! max number of iteration for point pos 

//! rinex.f90 -----------------------------------------------------------------
#define NUMSYS 7                           //! number of systems 
#define MAXRNXLEN (16*MAXOBSTYPE+4)        //! max rinex record length 
#define MAXPOSHEAD 1024                    //! max head line position 
#define MINFREQ_GLO -7                     //! min frequency number glonass 
#define MAXFREQ_GLO 13                     //! max frequency number glonass 
#define NINCOBS 262144/2                   //! inclimental number of obs data 

//! postpos.f90 ---------------------------------------------------------------
#define MAXINFILE 1000                     //! max number of input files 

//! rnx2rtkp.f90 --------------------------------------------------------------
#define MAXFILE 16                         //! max number of input files 
#define FLNUMLIM 200                       //! max number of rnxlist

extern const double PI;                    //! pi 
extern const double D2R;                   //! deg to rad 
extern const double R2D;                   //! rad to deg 
extern const double CLIGHT;                //! speed of light (m/s) 
extern const double SC2RAD;                //! semi-circle to radian (IS-GPS) 
extern const double AU;                    //! 1 AU (m) 
extern const double AS2R;                  //! arc sec to radian 

extern const double OMGE;                  //! earth angular velocity (IS-GPS) (rad/s)
extern const double RE_WGS84;              //! earth semimajor axis (WGS84) (m) 
extern const double FE_WGS84;              //! earth flattening (WGS84) 

extern const double HION;                  //! ionosphere height (m) 
                        

extern const double FREQL1;                //! L1/E1  frequency (Hz)
extern const double FREQL2;                //! L2     frequency (Hz)
extern const double FREQL5;                //! L5/E5a frequency (Hz)
extern const double FREQL6;                //! E6/LEX frequency (Hz)
extern const double FREQE5b;               //! E5b    frequency (Hz)
extern const double FREQE5ab;              //! E5a+b  frequency (Hz)
extern const double FREQs;                 //! S      frequency (Hz)
extern const double FREQ1_GLO;             //! GLONASS G1 base frequency (Hz)
extern const double DFRQ1_GLO;             //! GLONASS G1 bias frequency (Hz/n)
extern const double FREQ2_GLO;             //! GLONASS G2 base frequency (Hz)
extern const double DFRQ2_GLO;             //! GLONASS G2 bias frequency (Hz/n)
extern const double FREQ1a_GLO;            // GLONASS G1a frequency (Hz) 
extern const double FREQ2a_GLO;            // GLONASS G2a frequency (Hz)
extern const double FREQ3_GLO;             //! GLONASS G3 frequency (Hz)
extern const double FREQ1_CMP;             //! BeiDou B1 frequency (Hz)
extern const double FREQ2_CMP;             //! BeiDou B2 frequency (Hz)
extern const double FREQ3_CMP;             //! BeiDou B3 frequency (Hz)

extern const double EFACT_GPS;             //! error factor: GPS
extern const double EFACT_GLO;             //! error factor: GLONASS
extern const double EFACT_GAL;             //! error factor: Galileo
extern const double EFACT_QZS;             //! error factor: QZSS
extern const double EFACT_CMP;             //! error factor: BeiDou
extern const double EFACT_IRN;             //! error factor: IRNSS
extern const double EFACT_SBS;             //! error factor: SBAS



extern const double DTTOL;                 //! tolerance of time difference (s) 
extern const double MAXDTOE;               //! max time difference to GPS Toe (s) 
extern const double MAXDTOE_QZS;           //! max time difference to QZSS Toe (s) 
extern const double MAXDTOE_GAL;           //! max time difference to Galileo Toe (s) 
extern const double MAXDTOE_CMP;           //! max time difference to BeiDou Toe (s) 
extern const double MAXDTOE_GLO;           //! max time difference to GLONASS Toe (s) 
extern const double MAXDTOE_SBS;           //! max time difference to SBAS Toe (s) 
extern const double MAXDTOE_S;             //! max time difference to ephem toe (s) for other 
extern const double MAXGDOP;               //! max GDOP 

extern const double INT_SWAP_TRAC;         //! swap interval of trace file (s) 
extern const double INT_SWAP_STAT;         //! swap interval of solution status file (s) 


extern const double  MAXSBSAGEF;           //! max age of SBAS fast correction (s) 
extern const double  MAXSBSAGEL;           //! max age of SBAS long term corr (s) 


extern const double RNX2VER;               //! RINEX ver.2 default output version 
extern const double RNX3VER;               //! RINEX ver.3 default output version 



extern const char COMMENTH[2];             //! comment line indicator for solution 
extern const char MSG_DISCONN[13];         //! disconnect message



extern const double P2_5;                  //! 2^-5
extern const double P2_6;                  //! 2^-6
extern const double P2_11;                 //! 2^-11
extern const double P2_15;                 //! 2^-15
extern const double P2_17;                 //! 2^-17
extern const double P2_19;                 //! 2^-19
extern const double P2_20;                 //! 2^-20
extern const double P2_21;                 //! 2^-21
extern const double P2_23;                 //! 2^-23
extern const double P2_24;                 //! 2^-24
extern const double P2_27;                 //! 2^-27
extern const double P2_29;                 //! 2^-29
extern const double P2_30;                 //! 2^-30
extern const double P2_31;                 //! 2^-31
extern const double P2_32;                 //! 2^-32
extern const double P2_33;                 //! 2^-33
extern const double P2_35;                 //! 2^-35
extern const double P2_38;                 //! 2^-38
extern const double P2_39;                 //! 2^-39
extern const double P2_40;                 //! 2^-40
extern const double P2_43;                 //! 2^-43
extern const double P2_48;                 //! 2^-48
extern const double P2_50;                 //! 2^-50
extern const double P2_55;                 //! 2^-55



//! rtkcmn.f90 ----------------------------------------------------------------
extern const double MAX_VAR_EPH;           //! max variance eph to reject satellite (m^2) 
extern const double gpst0[6];              //! gps time reference 
extern const double gst0[6];               //! galileo system time reference 
extern const double bdt0[6];               //! beidou time reference 

extern const double lam_carr[MAXFREQ];

//! leap seconds (y,m,d,h,m,s,utc-gpst) 
extern const double leaps[19][7];

//! observation code strings 
extern const char obscodes[60][3];

//! 1:L1/E1, 2:L2/B1, 3:L5/E5a/L3, 4:L6/LEX/B3, 5:E5b/B2, 6:E5(a+b), 7:S
extern const int obsfreqs[60];

//! code priority table 
//! L1/E1; L2/B1; L5/E5a/L3; L6/LEX/B3; E5b/B2; E5(a+b); S
extern const char codepris[7][MAXFREQ][17];

//! ephemeris.f90 -------------------------------------------------------------
extern const double RE_GLO;                //! radius of earth (m)            ref (2) 
extern const double MU_GPS;                //! gravitational externant         ref (1) 
extern const double MU_GLO;                //! gravitational externant         ref (2) 
extern const double MU_GAL;                //! earth gravitational externant   ref (7) 
extern const double MU_CMP;                //! earth gravitational externant   ref (9) 
extern const double J2_GLO;                //! 2nd zonal harmonic of geopot   ref (2) 

extern const double OMGE_GLO;              //! earth angular velocity (rad/s) ref (2) 
extern const double OMGE_GAL;              //! earth angular velocity (rad/s) ref (7) 
extern const double OMGE_CMP;              //! earth angular velocity (rad/s) ref (9) 
extern const double SIN_5;                 //! dsin(-5.e0 deg) 
extern const double COS_5;                 //! dcos(-5.e0 deg) 

extern const double ERREPH_GLO;            //! error of glonass ephemeris (m) 
extern const double TSTEP;                 //! integration step glonass ephemeris (s) 
extern const double RTOL_KEPLER;           //! relative tolerance for Kepler equation 

extern const double DEFURASSR;             //! default accurary of ssr corr (m) 
extern const double MAXECORSSR;            //! max orbit correction of ssr (m) 
extern const double MAXCCORSSR;            //! max clock correction of ssr (m) 
extern const double MAXAGESSR;             //! max age of ssr orbit and clock (s) 
extern const double MAXAGESSR_HRCLK;       //! max age of ssr high-rate clock (s) 
extern const double STD_BRDCCLK;           //! error of broadcast clock (m) 
extern const double STD_GAL_NAPA;          //! error of galileo ephemeris for NAPA (m) 

extern const int eph_sel[6];               //! GPS,GLO,GAL,QZS,BDS,SBS (ephemeris selections)

//! pntpos.f90 ----------------------------------------------------------------
extern const double ERR_ION;               //! ionospheric delay std (m) 
extern const double ERR_TROP;              //! tropspheric delay std (m) 
extern const double ERR_SAAS;              //! saastamoinen model error std (m) 
extern const double ERR_BRDCI;             //! broadcast iono model error factor 
extern const double ERR_CBIAS;             //! code bias error std (m) 
extern const double REL_HUMI;              //! relative humidity for saastamoinen model 


extern const char systyps[8];              //! satellite system codes 
extern const char obstyps[5];              //! obs type codes 
extern const char frqtyps[8];              //! frequency codes 
extern const double ura_eph[16];           //! ura values (ref [3] 20.3.3.3.1.1) 

extern const double ura_nominal[16];       //! ura nominal values 



#endif