#include "decodeobs.h"
#include "str.h"
#include "obscode.h"

decode_obsh(FILE *fp, char *buff, double ver, int *tsys,
                        char tobs[][MAXOBSTYPE][4],obs_t*  obs, nav_t *nav, sta_t *sta)
{
    // default codes for unknown code 
    char defcodes[7][8]={
        "CWX    ",  // GPS: L125____
        "CC     ",  // GLO: L12_____
        "X XXXX ",  // GAL: L1_5678_
        "CXXX   ",  // QZS: L1256___
        "C X    ",  // SBS: L1_5____
        "XIXIIX ",  // BDS: L125678_
        "  A   A"};  // IRN: L__5___9
    double del[3];
    int i,j,k,n,nt,prn,fcn;
    char* p;
    char* label = buff + 60;
    char str[4];
    if (strstr(label, "MARKER NUMBER"))
    {
        if(sta) setstr(sta->name, buff, 60);
    }
    else if (strstr(label, "MARKER NUMBER"))
    {
        if(sta) setstr(sta->marker, buff, 20);
    }
    else if (strstr(label, "MARKER TYPE"));
    else if (strstr(label, "OBSERVER / AGENCY"));
    else if (strstr(label, "REC # / TYPE / VERS"))
    {
        if(sta)
        {
            setstr(sta->recsno, buff, 20);
            setstr(sta->rectype, buff + 20, 20);
            setstr(sta->recver, buff + 40, 20);
        }
    }
    else if (strstr(label, "ANT # / TYPE"))
    {
        if (sta)
        {
            setstr(sta->antsno, buff, 20);
            setstr(sta->antdes, buff + 20, 20);
        }
    }
    else if (strstr(label, "APPROX POSITION XYZ"))
    {
        if (sta)
        {
            for (i = 0, j = 0; i < 3; i++, j += 14)
            {
                sta->pos[i] = str2num(buff, j, 14);
            }
                
        }
    }
    else if (strstr(label, "ANTENNA: DELTA H/E/N"))
    {
        if (sta)
        {
            for (i = 0, j = 0; i < 3; i++, j += 14)
            {
                del[i] = str2num(buff, j, 14);
            }
            sta->del[2] = del[0]; /* h */
            sta->del[0] = del[1]; /* e */
            sta->del[1] = del[2]; /* n */
        }
    }
    else if(strstr(label,"ANTENNA: DELTA X/Y/Z"));  // opt ver.3 
    else if(strstr(label,"ANTENNA: PHASECENTER"));  // opt ver.3 
    else if(strstr(label,"ANTENNA: B.SIGHT XYZ"));  // opt ver.3 
    else if(strstr(label,"ANTENNA: ZERODIR AZI"));  // opt ver.3 
    else if(strstr(label,"ANTENNA: ZERODIR XYZ"));  // opt ver.3 
    else if(strstr(label,"CENTER OF MASS: XYZ" ));  // opt ver.3 
    else if(strstr(label,"SYS / # / OBS TYPES" ))  // ver.3 
    {
        if (!(p = strchr(systyps, buff[0])))
        {
            printf("invalid system code: sys=%c\n", buff[0]);
            return;
        }
        i = (int)(p - systyps);
        n = (int)str2num(buff, 3, 3);
        for (j = nt = 0, k = 7; j < n; j++, k += 4)
        {
            if (k > 58)
            {
                if (!fgets(buff, MAXRNXLEN, fp))
                    break;
                k = 7;
            }
            if (nt < MAXOBSTYPE - 1)
                setstr(tobs[i][nt++], buff + k, 3);
        }
        *tobs[i][nt] = '\0';

        /* change BDS B1 code: 3.02 */
        if (i == 5 && fabs(ver - 3.02) < 1e-3)
        {
            for (j = 0; j < nt; j++)
                if (tobs[i][j][1] == '1')
                    tobs[i][j][1] = '2';
        }

        for (j=0;j<nt;j++) {
            if (tobs[i][j][2]) continue;
            if (!(p=strchr(frqtyps,tobs[i][j][1]))) continue;
            tobs[i][j][2]=defcodes[i][(int)(p-frqtyps)];
            printf("set default for unknown code: sys=%c code=%s\n",buff[0],
                  tobs[i][j]);
        }
    }
    else if (strstr(label, "WAVELENGTH FACT L1/2")); /* opt ver.2 */
    else if (strstr(label, "# / TYPES OF OBSERV"))
    { /* ver.2 */
        n = (int)str2num(buff, 0, 6);
        for (i = nt = 0, j = 10; i < n; i++, j += 6)
        {
            if (j > 58)
            {
                if (!fgets(buff, MAXRNXLEN, fp))
                    break;
                j = 10;
            }
            if (nt >= MAXOBSTYPE - 1)
                continue;
            if (ver <= 2.99)
            {
                setstr(str, buff + j, 2);
                convcode(ver, SYS_GPS, str, tobs[0][nt]);
                convcode(ver, SYS_GLO, str, tobs[1][nt]);
                convcode(ver, SYS_GAL, str, tobs[2][nt]);
                convcode(ver, SYS_QZS, str, tobs[3][nt]);
                convcode(ver, SYS_SBS, str, tobs[4][nt]);
                convcode(ver, SYS_CMP, str, tobs[5][nt]);
            }
            nt++;
        }
        *tobs[0][nt] = '\0';
    }
    else if (strstr(label, "SIGNAL STRENGTH UNIT")); /* opt ver.3 */
    else if (strstr(label, "INTERVAL"))
    {
        obs->tint = str2num(buff,0,11);
    } 
    else if (strstr(label, "TIME OF FIRST OBS"))
    {
        if (!strncmp(buff + 48, "GPS", 3))
            *tsys = TSYS_GPS;
        else if (!strncmp(buff + 48, "GLO", 3))
            *tsys = TSYS_UTC;
        else if (!strncmp(buff + 48, "GAL", 3))
            *tsys = TSYS_GAL;
        else if (!strncmp(buff + 48, "QZS", 3))
            *tsys = TSYS_QZS; /* ver.3.02 */
        else if (!strncmp(buff + 48, "BDT", 3))
            *tsys = TSYS_CMP; /* ver.3.02 */
        else if (!strncmp(buff + 48, "IRN", 3))
            *tsys = TSYS_IRN; /* ver.3.03 */
    }
    else if (strstr(label, "TIME OF LAST OBS"));
    else if (strstr(label, "RCV CLOCK OFFS APPL"));
    else if (strstr(label, "SYS / DCBS APPLIED"));
    else if (strstr(label, "SYS / PCVS APPLIED"));
    else if (strstr(label, "SYS / SCALE FACTOR"));
    else if (strstr(label, "SYS / PHASE SHIFTS")); /* ver.3.01 */
    else if (strstr(label, "GLONASS SLOT / FRQ #"))
    { /* ver.3.02 */
        for (i = 0; i < 8; i++)
        {
            if (buff[4 + i * 7] != 'R')
                continue;
            prn = (int)str2num(buff, 5 + i * 7, 2);
            fcn = (int)str2num(buff, 8 + i * 7, 2);
            if (prn < 1 || prn > MAXPRNGLO || fcn < -7 || fcn > 6)
                continue;
            if (nav)
                nav->glo_fcn[prn - 1] = fcn + 8;
        }
    }
    else if (strstr(label, "GLONASS COD/PHS/BIS"))
    { /* ver.3.02 */
        if (nav)
        {
            nav->glo_cpbias[0] = str2num(buff, 5, 8);
            nav->glo_cpbias[1] = str2num(buff, 18, 8);
            nav->glo_cpbias[2] = str2num(buff, 31, 8);
            nav->glo_cpbias[3] = str2num(buff, 44, 8);
        }
    }
    else if (strstr(label, "LEAP SECONDS"))
    { /* opt */
        if (nav)
        {
            /*nav->utc_gps[4] = str2num(buff, 0, 6);
            nav->utc_gps[7] = str2num(buff, 6, 6);
            nav->utc_gps[5] = str2num(buff, 12, 6);
            nav->utc_gps[6] = str2num(buff, 18, 6);*/
            nav->leaps = (int)str2num(buff,0,6);
        }
    }
    else if (strstr(label, "# OF SATELLITES"));
    else if (strstr(label, "PRN / # OF OBS"));
    return;
}

int decode_obsepoch(FILE *fp, char *buff, double ver, gtime_t *time,
                           int *flag, int *sats)
{
    int i, j, n;
    char satid[8] = {'\0'};

    printf("decode_obsepoch: ver=%.2f\n", ver);

    if (ver <= 2.99)
    { /* ver.2 */
        /* epoch flag: 3:new site,4:header info,5:external event */
        *flag = (int)str2num(buff, 28, 1);

        /* handle external event */
        if (*flag == 5)
        {
            str2time(buff, 0, 26, time);
        }

        if ((n = (int)str2num(buff, 29, 3)) <= 0)
            return 0;

        if (3 <= *flag && *flag <= 5)
            return n;

        if (str2time(buff, 0, 26, time))
        {
            printf("rinex obs invalid epoch: epoch=%26.26s\n", buff);
            return 0;
        }
        for (i = 0, j = 32; i < n; i++, j += 3)
        {
            if (j >= 68)
            {
                if (!fgets(buff, MAXRNXLEN, fp))
                    break;
                j = 32;
            }
            if (i < MAXOBS)
            {
                strncpy(satid, buff + j, 3);
                sats[i] = satid2no(satid);
            }
        }
    }
    else
    { /* ver.3 */
        *flag = (int)str2num(buff, 31, 1);

        /* handle external event */
        if (*flag == 5)
        {
            str2time(buff, 1, 28, time);
        }

        if ((n = (int)str2num(buff, 32, 3)) <= 0)
            return 0;

        if (3 <= *flag && *flag <= 5)
            return n;

        if (buff[0] != '>' || str2time(buff, 1, 28, time))
        {
            printf("rinex obs invalid epoch: epoch=%29.29s\n", buff);
            return 0;
        }
    }
    char time_str[64];
    time2str(*time, time_str, 3);

    printf("decode_obsepoch: time=%s flag=%d\n", time_str, *flag);
    return n;
}

int decode_obsdata(FILE *fp, char *buff, double ver, int mask,
                          sigind_t *index, obsd_t *obs)
{
    sigind_t *ind;
    double val[MAXOBSTYPE] = {0};
    unsigned char lli[MAXOBSTYPE] = {0};
    unsigned char std[MAXOBSTYPE] = {0};
    char satid[8] = "";
    int i, j, n, m, q, status = 1, p[MAXOBSTYPE], k[16], l[16], r[16];

    if (ver > 2.99)
    { /* ver.3 */
        sprintf(satid, "%.3s", buff);
        obs->sat = (unsigned char)satid2no(satid);
    }
    if (!obs->sat)
    {
        printf("decode_obsdata: unsupported sat sat=%s\n", satid);
        status = 0;
    }
    else if (!(satsys(obs->sat, NULL) & mask))
    {
        status = 0;
    }
    /* read observation data fields */
    switch (satsys(obs->sat, NULL))
    {
    case SYS_GLO:
        ind = index + 1;
        break;
    case SYS_GAL:
        ind = index + 2;
        break;
    case SYS_QZS:
        ind = index + 3;
        break;
    case SYS_SBS:
        ind = index + 4;
        break;
    case SYS_CMP:
        ind = index + 5;
        break;
    case SYS_IRN:
        ind = index + 6;
        break;
    default:
        ind = index;
        break;
    }
    for (i = 0, j = ver <= 2.99 ? 0 : 3; i < ind->n; i++, j += 16)
    {

        if (ver <= 2.99 && j >= 80)
        { /* ver.2 */
            if (!fgets(buff, MAXRNXLEN, fp))
                break;
            j = 0;
        }
        if (status)
        {
            val[i] = str2num(buff, j, 14) + ind->shift[i];
            lli[i] = (unsigned char)str2num(buff, j + 14, 1) & 3;
            /* measurement std from receiver */
            std[i] = (unsigned char)str2num(buff, j + 15, 1);
        }
    }
    if (!status)
        return 0;

    for (i = 0; i < NFREQ + NEXOBS; i++)
    {
        obs->P[i] = obs->L[i] = 0.0;
        obs->D[i] = 0.0f;
        obs->SNR[i] = obs->LLI[i] = obs->qualL[i] = obs->qualP[i] = obs->code[i] = 0;
    }
    /* assign position in observation data */
    for (i = n = m = q = 0; i < ind->n; i++)
    {

        p[i] = ind->frqIdx[i];

        if (ind->type[i] == 0 && p[i] == 0)
            k[n++] = i; /* C1? index */
        if (ind->type[i] == 0 && p[i] == 1)
            l[m++] = i; /* C2? index */
        if (ind->type[i] == 0 && p[i] == 2)
            r[q++] = i; /* C3? index */
    }

    /* if multiple codes (C1/P1,C2/P2), select higher priority */
    if (n >= 2)
    {
        if (val[k[0]] == 0.0 && val[k[1]] == 0.0)
        {
            p[k[0]] = -1;
            p[k[1]] = -1;
        }
        else if (val[k[0]] != 0.0 && val[k[1]] == 0.0)
        {
            p[k[0]] = 0;
            p[k[1]] = -1;
        }
        else if (val[k[0]] == 0.0 && val[k[1]] != 0.0)
        {
            p[k[0]] = -1;
            p[k[1]] = 0;
        }
        else if (ind->pri[k[1]] > ind->pri[k[0]])
        {
            p[k[1]] = 0;
            p[k[0]] = NEXOBS < 1 ? -1 : NFREQ;
        }
        else
        {
            p[k[0]] = 0;
            p[k[1]] = NEXOBS < 1 ? -1 : NFREQ;
        }
    }
    if (m >= 2)
    {
        if (val[l[0]] == 0.0 && val[l[1]] == 0.0)
        {
            p[l[0]] = -1;
            p[l[1]] = -1;
        }
        else if (val[l[0]] != 0.0 && val[l[1]] == 0.0)
        {
            p[l[0]] = 1;
            p[l[1]] = -1;
        }
        else if (val[l[0]] == 0.0 && val[l[1]] != 0.0)
        {
            p[l[0]] = -1;
            p[l[1]] = 1;
        }
        else if (ind->pri[l[1]] > ind->pri[l[0]])
        {
            p[l[1]] = 1;
            p[l[0]] = NEXOBS < 2 ? -1 : NFREQ + 1;
        }
        else
        {
            p[l[0]] = 1;
            p[l[1]] = NEXOBS < 2 ? -1 : NFREQ + 1;
        }
    }
    if (q >= 2)
    {
        if (val[r[0]] == 0.0 && val[r[1]] == 0.0)
        {
            p[r[0]] = -1;
            p[r[1]] = -1;
        }
        else if (val[r[0]] != 0.0 && val[r[1]] == 0.0)
        {
            p[r[0]] = 2;
            p[r[1]] = -1;
        }
        else if (val[r[0]] == 0.0 && val[r[1]] != 0.0)
        {
            p[r[0]] = -1;
            p[r[1]] = 2;
        }
        else if (ind->pri[r[1]] > ind->pri[r[0]])
        {
            p[r[1]] = 2;
            p[r[0]] = NEXOBS < 3 ? -1 : NFREQ + 2;
        }
        else
        {
            p[r[0]] = 2;
            p[r[1]] = NEXOBS < 3 ? -1 : NFREQ + 2;
        }
    }
    /* save observation data */
    for (i = 0; i < ind->n; i++)
    {
        if (p[i] < 0 || (val[i] == 0.0 && lli[i] == 0))
            continue;
        switch (ind->type[i])
        {
        case 0:
            obs->P[p[i]] = val[i];
            obs->code[p[i]] = ind->code[i];
            obs->qualP[p[i]] = std[i] > 0 ? std[i] : 1;
            break;
        case 1:
            obs->L[p[i]] = val[i];
            obs->LLI[p[i]] = lli[i];
            obs->qualL[p[i]] = std[i] > 0 ? std[i] : 1;
            break;
        case 2:
            obs->D[p[i]] = (float)val[i];
            break;
        case 3:
            obs->SNR[p[i]] = (unsigned char)(val[i] / SNR_UNIT + 0.5);
            break;
        }
        printf("obs: i=%d f=%d P=%10.3f L=%10.3f LLI=%d code=%d\n", i, p[i], obs->P[p[i]],
              obs->L[p[i]], obs->LLI[p[i]], obs->code[p[i]]);
    }
    char time_str[64];
    time2str(obs->time, time_str, 0);
    printf("decode_obsdata: time=%s sat=%2d\n", time_str, obs->sat);
    return 1;
}