#include "obscode.h"
#include "const.h"

void convcode(double ver, int sys, const char *str, char *type)
{
    strcpy(type, "   ");

    if (!strcmp(str, "P1"))
    { /* ver.2.11 GPS L1PY,GLO L2P */
        if (sys == SYS_GPS)
            sprintf(type, "%c1W", 'C');
        else if (sys == SYS_GLO)
            sprintf(type, "%c1P", 'C');
    }
    else if (!strcmp(str, "P2"))
    { /* ver.2.11 GPS L2PY,GLO L2P */
        if (sys == SYS_GPS)
            sprintf(type, "%c2W", 'C');
        else if (sys == SYS_GLO)
            sprintf(type, "%c2P", 'C');
    }
    else if (!strcmp(str, "C1"))
    { /* ver.2.11 GPS L1C,GLO L1C/A */
        if (ver >= 2.12)
            ; /* reject C1 for 2.12 */
        else if (sys == SYS_GPS)
            sprintf(type, "%c1C", 'C');
        else if (sys == SYS_GLO)
            sprintf(type, "%c1C", 'C');
        else if (sys == SYS_GAL)
            sprintf(type, "%c1X", 'C'); /* ver.2.12 */
        else if (sys == SYS_QZS)
            sprintf(type, "%c1C", 'C');
        else if (sys == SYS_SBS)
            sprintf(type, "%c1C", 'C');
    }
    else if (!strcmp(str, "C2"))
    {
        if (sys == SYS_GPS)
        {
            if (ver >= 2.12)
                sprintf(type, "%c2W", 'C'); /* L2P(Y) */
            else
                sprintf(type, "%c2X", 'C'); /* L2C */
        }
        else if (sys == SYS_GLO)
            sprintf(type, "%c2C", 'C');
        else if (sys == SYS_QZS)
            sprintf(type, "%c2X", 'C');
        else if (sys == SYS_CMP)
            sprintf(type, "%c2X", 'C'); /* ver.2.12 B1_2 */
    }
    else if (ver >= 2.12 && str[1] == 'A')
    { /* ver.2.12 L1C/A */
        if (sys == SYS_GPS)
            sprintf(type, "%c1C", str[0]);
        else if (sys == SYS_GLO)
            sprintf(type, "%c1C", str[0]);
        else if (sys == SYS_QZS)
            sprintf(type, "%c1C", str[0]);
        else if (sys == SYS_SBS)
            sprintf(type, "%c1C", str[0]);
    }
    else if (ver >= 2.12 && str[1] == 'B')
    { /* ver.2.12 GPS L1C */
        if (sys == SYS_GPS)
            sprintf(type, "%c1X", str[0]);
        else if (sys == SYS_QZS)
            sprintf(type, "%c1X", str[0]);
    }
    else if (ver >= 2.12 && str[1] == 'C')
    { /* ver.2.12 GPS L2C */
        if (sys == SYS_GPS)
            sprintf(type, "%c2X", str[0]);
        else if (sys == SYS_QZS)
            sprintf(type, "%c2X", str[0]);
    }
    else if (ver >= 2.12 && str[1] == 'D')
    { /* ver.2.12 GLO L2C/A */
        if (sys == SYS_GLO)
            sprintf(type, "%c2C", str[0]);
    }
    else if (ver >= 2.12 && str[1] == '1')
    { /* ver.2.12 GPS L1PY,GLO L1P */
        if (sys == SYS_GPS)
            sprintf(type, "%c1W", str[0]);
        else if (sys == SYS_GLO)
            sprintf(type, "%c1P", str[0]);
        else if (sys == SYS_GAL)
            sprintf(type, "%c1X", str[0]); /* tentative */
        else if (sys == SYS_CMP)
            sprintf(type, "%c2X", str[0]); /* extension */
    }
    else if (ver < 2.12 && str[1] == '1')
    {
        if (sys == SYS_GPS)
            sprintf(type, "%c1C", str[0]);
        else if (sys == SYS_GLO)
            sprintf(type, "%c1C", str[0]);
        else if (sys == SYS_GAL)
            sprintf(type, "%c1X", str[0]); /* tentative */
        else if (sys == SYS_QZS)
            sprintf(type, "%c1C", str[0]);
        else if (sys == SYS_SBS)
            sprintf(type, "%c1C", str[0]);
    }
    else if (str[1] == '2')
    {
        if (sys == SYS_GPS)
            sprintf(type, "%c2W", str[0]);
        else if (sys == SYS_GLO)
            sprintf(type, "%c2P", str[0]);
        else if (sys == SYS_QZS)
            sprintf(type, "%c2X", str[0]);
        else if (sys == SYS_CMP)
            sprintf(type, "%c2X", str[0]); /* ver.2.12 B1_2 */
    }
    else if (str[1] == '5')
    {
        if (sys == SYS_GPS)
            sprintf(type, "%c5X", str[0]);
        else if (sys == SYS_GAL)
            sprintf(type, "%c5X", str[0]);
        else if (sys == SYS_QZS)
            sprintf(type, "%c5X", str[0]);
        else if (sys == SYS_SBS)
            sprintf(type, "%c5X", str[0]);
    }
    else if (str[1] == '6')
    {
        if (sys == SYS_GAL)
            sprintf(type, "%c6X", str[0]);
        else if (sys == SYS_QZS)
            sprintf(type, "%c6X", str[0]);
        else if (sys == SYS_CMP)
            sprintf(type, "%c6X", str[0]); /* ver.2.12 B3 */
    }
    else if (str[1] == '7')
    {
        if (sys == SYS_GAL)
            sprintf(type, "%c7X", str[0]);
        else if (sys == SYS_CMP)
            sprintf(type, "%c7X", str[0]); /* ver.2.12 B2b */
    }
    else if (str[1] == '8')
    {
        if (sys == SYS_GAL)
            sprintf(type, "%c8X", str[0]);
    }
    printf("convcode: ver=%.2f sys=%2d type= %s -> %s\n", ver, sys, str, type);
}