#ifndef OPTION_H
#define OPTION_H
#include "const.h"
#include "comtyp.h"
#include <stdbool.h>

typedef struct 
{
    int mode;                // positioning mode (PMODE_???) 
    int soltype;        // solution type (0:forward,1:backward,2:combined) 
    int nf;             // number of frequencies (1:L1,2:L1+L2,3:L1+L2+L5) 
    int navsys;         // navigation system 
    double elmin;             // elevation mask angle (rad) 
    int sateph;         // satellite ephemeris/clock (EPHOPT_???) 
    int ionoopt;        // ionosphere option (IONOOPT_???) 
    int tropopt;        // troposphere option (TROPOPT_???) 
    int niter;          // number of filter iteration 
    double maxinno;           // reject threshold of innovation (m) 
    double maxgdop;           // reject threshold of gdop 
    int exsats[MAXSAT]; // excluded satellites (1:excluded,2:included)
    char rnxopt[2][256]; // rinex options {rover,base}
    int posopt[6];      // positioning options
    bool lsa;              // SA data (before 2000/123)
}prcopt_t;

typedef struct 
{
    int posf;           // solution format (SOLF_???) 
    int times;          // time system (TIMES_???) 
    int timef;          // time format (0:sssss.s,1:yyyy/mm/dd hh:mm:ss.s) 
    int timeu;          // time digits under decimal point 
    int degf;           // latitude/longitude format (0:ddd.ddd,1:ddd mm ss) 
    int outhead;        // output header (0:no,1:yes) 
    int outopt;         // output processing options (0:no,1:yes) 
    int outvel;         // output velocity options (0:no,1:yes) 
    int datum;          // datum (0:WGS84,1:Tokyo) 
    int height;         // height (0:ellipsoidal,1:geodetic) 
    int geoid;          // geoid model (0:EGM96,1:JGD2000) 
    int solstatic;      // solution of static mode (0:all,1:single) 
    int sstat;          // solution statistics level (0:off,1:states,2:residuals) 
    int trace;          // debug trace level (0:off,1-5:debug) 
    int issingle;       // single solution (0:no,1:yes)
    double nmeaintv[2];       // nmea output interval (s) ( .lt. 0:no,0:all) 
                             // nmeaintv(0):gprmc,gpgga,nmeaintv(1):gpgsv 
    char sep[64];        // field separator 
    char prog[64];       // program name 
    double maxsolstd;         // max std-dev for solution output (m) (0:all) 
}solopt_t;

typedef struct 
{
    gtime_t ts,te;      // time start/end 
    double tint;              // time interval (s) 
    double ttol;              // time tolerance (s) 
    double tunit;             // time unit for multiple-session (s) 
    double rnxver;            // RINEX version 
    int navsys;         // navigation system 
    int obstype;        // observation type 
    int freqtype;      // frequency type 
    char mask[7][64];    // code mask {GPS,GLO,GAL,QZS,SBS,CMP,IRN} 
    char staid[32];      // station id for rinex file name 
    char prog[32];       // program 
    char runby[32];      // run-by 
    char marker[32];     // marker name 
    char markerno[32];   // marker number 
    char markertype[32]; // marker type (ver.3) 
    char name[2][32];    // observer/agency 
    char rec[3][32];    // receiver #/type/vers 
    char ant[3][32];    // antenna #/type 
    int exsats[MAXSAT]; // excluded satellites 
    gtime_t tstart;     // first obs time 
    gtime_t tend;       // last obs time 
    char tobs[7][MAXOBSTYPE][4]; // obs types {GPS,GLO,GAL,QZS,SBS,CMP,IRN} 
    int nobs[7];        // number of obs types {GPS,GLO,GAL,QZS,SBS,CMP,IRN}
}rnxopt_t;


#endif