#ifndef READOBSNAV_H
#define READOBSNAV_H

#include "comtyp.h"
#include "option.h"
#include "const.h"

// Read obs and nav data -----------------------------------------------------
bool readobsnav(gtime_t ts, gtime_t te, double ti,const char** infile,const int* index,int n,
                const prcopt_t* prcopt,obs_t* obs,nav_t* nav,sta_t* sta);

// read rinex obs and nav files ----------------------------------------------
//return -1,0: error, 1-normal
int readrnxt(const char *filepath,int rcv,gtime_t ts,gtime_t te,double ti,const char* opt,
                obs_t* obs,nav_t* nav,sta_t* sta);

//read rinex file --------------------------------------------
//return -1,0: error, 1-normal
int readrnxfile(const char *filepath,gtime_t ts,gtime_t te,double ti,const char* opt, 
                int flag,int index,char* type,obs_t* obs,nav_t* nav,sta_t* sta);

// read rinex file -----------------------------------------------------------
// 0-error, 1-normal
int readrnxfp(FILE* fp,gtime_t ts,gtime_t te,double ti,const char* opt, 
                    int flag,int index,char* type,obs_t*  obs,nav_t* nav,sta_t* sta);

/* read RINEX file header ----------------------------------------------------*/
// 0-error, 1-normal
int readrnxh(FILE *fp, double *ver, char *type, int *sys, int *tsys,
             char tobs[][MAXOBSTYPE][4],obs_t*  obs, nav_t *nav, sta_t *sta);

/* read RINEX observation data -----------------------------------------------*/
int readrnxobs(FILE *fp, gtime_t ts, gtime_t te, double ti,
                      const char *opt, int rcv, double ver, int *tsys,
                      char tobs[][MAXOBSTYPE][4], obs_t *obs, sta_t *sta);

/* read RINEX observation data body ------------------------------------------*/
int readrnxobsb(FILE *fp, const char *opt, double ver, int *tsys,
                       char tobs[][MAXOBSTYPE][4], int *flag, obsd_t *data,
                       sta_t *sta);

/* save cycle slips ----------------------------------------------------------*/
void saveslips(unsigned char slips[][NFREQ + NEXOBS], obsd_t *data);

/* screen by time --------------------------------------------------------------
 * screening by time start, time end, and time interval
 * args   : gtime_t time  I      time
 *          gtime_t ts    I      time start (ts.time==0:no screening by ts)
 *          gtime_t te    I      time end   (te.time==0:no screening by te)
 *          double  tint  I      time interval (s) (0.0:no screen by tint)
 * return : 1:on condition, 0:not on condition
 *-----------------------------------------------------------------------------*/
int screent(gtime_t time, gtime_t ts, gtime_t te, double tint);

/* restore cycle slips -------------------------------------------------------*/
void restoreslips(unsigned char slips[][NFREQ + NEXOBS], obsd_t *data);

/* add observation data ------------------------------------------------------*/
int addobsdata(obs_t *obs, const obsd_t *data);

/* read RINEX navigation data ------------------------------------------------*/
int readrnxnav(FILE *fp, const char *opt, double ver, int sys, nav_t *nav);

/* read RINEX navigation data body -------------------------------------------*/
int readrnxnavb(FILE *fp, const char *opt, double ver, int sys,
                       int *type, eph_t *eph, geph_t *geph, seph_t *seph);

void freeobsnav(obs_t *obs, nav_t* nav);

#endif