#include "satellite.h"
#include "const.h"

extern int satid2no(const char *id)
{
    int sys, prn;
    char code;

    if (sscanf(id, "%d", &prn) == 1)
    {
        if (MINPRNGPS <= prn && prn <= MAXPRNGPS)
            sys = SYS_GPS;
        else if (MINPRNSBS <= prn && prn <= MAXPRNSBS)
            sys = SYS_SBS;
        else if (MINPRNQZS <= prn && prn <= MAXPRNQZS)
            sys = SYS_QZS;
        else
            return 0;
        return satno(sys, prn);
    }
    if (sscanf(id, "%c%d", &code, &prn) < 2)
        return 0;

    switch (code)
    {
    case 'G':
        sys = SYS_GPS;
        prn += MINPRNGPS - 1;
        break;
    case 'R':
        sys = SYS_GLO;
        prn += MINPRNGLO - 1;
        break;
    case 'E':
        sys = SYS_GAL;
        prn += MINPRNGAL - 1;
        break;
    case 'J':
        sys = SYS_QZS;
        prn += MINPRNQZS - 1;
        break;
    case 'C':
        sys = SYS_CMP;
        prn += MINPRNCMP - 1;
        break;
    case 'I':
        sys = SYS_IRN;
        prn += MINPRNIRN - 1;
        break;
    case 'L':
        sys = SYS_LEO;
        prn += MINPRNLEO - 1;
        break;
    case 'S':
        sys = SYS_SBS;
        prn += 100;
        break;
    default:
        return 0;
    }
    return satno(sys, prn);
}

int satno(int sys, int prn)
{
    if (prn <= 0)
        return 0;
    switch (sys)
    {
    case SYS_GPS:
        if (prn < MINPRNGPS || MAXPRNGPS < prn)
            return 0;
        return prn - MINPRNGPS + 1;
    case SYS_GLO:
        if (prn < MINPRNGLO || MAXPRNGLO < prn)
            return 0;
        return NSATGPS + prn - MINPRNGLO + 1;
    case SYS_GAL:
        if (prn < MINPRNGAL || MAXPRNGAL < prn)
            return 0;
        return NSATGPS + NSATGLO + prn - MINPRNGAL + 1;
    case SYS_QZS:
        if (prn < MINPRNQZS || MAXPRNQZS < prn)
            return 0;
        return NSATGPS + NSATGLO + NSATGAL + prn - MINPRNQZS + 1;
    case SYS_CMP:
        if (prn < MINPRNCMP || MAXPRNCMP < prn)
            return 0;
        return NSATGPS + NSATGLO + NSATGAL + NSATQZS + prn - MINPRNCMP + 1;
    case SYS_IRN:
        if (prn < MINPRNIRN || MAXPRNIRN < prn)
            return 0;
        return NSATGPS + NSATGLO + NSATGAL + NSATQZS + NSATCMP + prn - MINPRNIRN + 1;
    case SYS_LEO:
        if (prn < MINPRNLEO || MAXPRNLEO < prn)
            return 0;
        return NSATGPS + NSATGLO + NSATGAL + NSATQZS + NSATCMP + NSATIRN +
               prn - MINPRNLEO + 1;
    case SYS_SBS:
        if (prn < MINPRNSBS || MAXPRNSBS < prn)
            return 0;
        return NSATGPS + NSATGLO + NSATGAL + NSATQZS + NSATCMP + NSATIRN + NSATLEO +
               prn - MINPRNSBS + 1;
    }
    return 0;
}

int satsys(int sat, int *prn)
{
    int sys = SYS_NONE;
    if (sat <= 0 || MAXSAT < sat)
        sat = 0;
    else if (sat <= NSATGPS)
    {
        sys = SYS_GPS;
        sat += MINPRNGPS - 1;
    }
    else if ((sat -= NSATGPS) <= NSATGLO)
    {
        sys = SYS_GLO;
        sat += MINPRNGLO - 1;
    }
    else if ((sat -= NSATGLO) <= NSATGAL)
    {
        sys = SYS_GAL;
        sat += MINPRNGAL - 1;
    }
    else if ((sat -= NSATGAL) <= NSATQZS)
    {
        sys = SYS_QZS;
        sat += MINPRNQZS - 1;
    }
    else if ((sat -= NSATQZS) <= NSATCMP)
    {
        sys = SYS_CMP;
        sat += MINPRNCMP - 1;
    }
    else if ((sat -= NSATCMP) <= NSATIRN)
    {
        sys = SYS_IRN;
        sat += MINPRNIRN - 1;
    }
    else if ((sat -= NSATIRN) <= NSATLEO)
    {
        sys = SYS_LEO;
        sat += MINPRNLEO - 1;
    }
    else if ((sat -= NSATLEO) <= NSATSBS)
    {
        sys = SYS_SBS;
        sat += MINPRNSBS - 1;
    }
    else
        sat = 0;
    if (prn)
        *prn = sat;
    return sys;
}